/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
define([
    'jquery',
    'domReady',
    'matchMedia',
    'jquery/ui'
], function ($, domReady, mediaCheck) {
    'use strict';

    $.widget('filoblu.layout', {
        options: {
            responsive: false,
            disabled: false
        },

        _create: function(){
            var self = this,
                lastScrollTop = 0,
                headerTransparent = $('header.page-header').hasClass('transparent'),
                logoLeft = $('header.page-header').hasClass('left-logo'),
                isHomePage = $('body').hasClass('cms-index-index');
            self._super();

            if (this.options.responsive === true) {

                mediaCheck({
                    media: '(max-width: 991px)',
                    entry: $.proxy(function () {
                        self.moveBlocks('mobile',logoLeft);
                        self.calcHeaderHeight();
                        self.calcMobileMenuHeight();
                        if (!headerTransparent){
                            self.setMainContainerTopPadding('mobile');
                            $(window).bind('touchmove', function(){
                                self.stickyHeaderAnimation();
                                self.chekIfinView();
                            });
                            //$(window).unbind('scroll');
                            //$(window).unbind('mousemove');
                        } else if (headerTransparent) {
                            if (!isHomePage) {
                                self.setMainContainerTopPadding('mobile');
                            }
                            $(window).bind('touchmove', function(){
                                self.transparentHeaderToPlainAnimation();
                                self.chekIfinView();
                            });
                            //$(window).unbind('scroll');
                        }
                    }, this),
                    exit: $.proxy(function () {
                        self.moveBlocks('desktop',logoLeft);
                        self.calcHeaderHeight();
                        if (!headerTransparent){
                            self.setMainContainerTopPadding('desktop');
                            // $(window).bind('mousemove', function(){
                            //     self.headerOnMouseMove(event);
                            // });
                            $(window).bind('mousewheel DOMMouseScroll', function(){
                                self.stickyHeaderAnimation();
                                self.chekIfinView();
                            });
                            //$(window).unbind('touchmove');
                        } else if (headerTransparent) {
                            if (!isHomePage) {
                                self.setMainContainerTopPadding('mobile');
                            }
                            $(window).bind('mousewheel DOMMouseScroll', function(){
                                self.transparentHeaderToPlainAnimation();
                                self.chekIfinView();
                                self.stickyHeaderAnimation();
                            });
                            //$(window).unbind('touchmove');
                        }
                    }, this)
                });

                domReady(function () {
                    window.lastScrollTop = 0;
                    window.headerHeight = 0;
                    self.chekIfinView();
                });

            }


        },

        setMainContainerTopPadding: function (mode) {
            if (mode === 'mobile') {
                $('#maincontent').css('margin-top', window.headerHeight);
            } else if (mode === 'desktop') {
                $('#maincontent').css('margin-top', window.headerHeight);
            }
        },

        //@ToDo This function has been update just to deliver sticky class
        stickyHeaderAnimation: function(event) {
            var st = $(window).scrollTop(),
                header = $('header.page-header');

            if (!this.isMobileMenuOpen()) {
                if (st > 0){
                    // if (!header.hasClass('pulled-up')) {
                    //     header.addClass('pulled-up');
                    // }
                    if (!header.hasClass('sticky-header')) {
                        header.addClass('sticky-header');
                    }
                } else {
                    // if (header.hasClass('pulled-up')) {
                    //     header.removeClass('pulled-up');
                    // }
                    if (header.hasClass('sticky-header')) {
                        header.removeClass('sticky-header');
                    }
                }
            }

        },

        transparentHeaderToPlainAnimation: function(event) {
            var st = $(window).scrollTop(),
                header = $('header.page-header');

            if (!this.isMobileMenuOpen()) {
                if (st > 0){
                    if (!header.hasClass('plain')) {
                        header.addClass('plain');
                    }
                } else {
                    if (header.hasClass('plain')) {
                        header.removeClass('plain');
                    }
                }
            }

        },

        //@ToDo remove due to some bugs, no more implemented
        headerOnMouseMove: function(event) {
            var mouseY = event.clientY,
                header = $('header.page-header');
            if (mouseY <= 50) {
                if (header.hasClass('pulled-up')) {
                    header.removeClass('pulled-up');
                }
            }
        },

        isMobileMenuOpen: function () {
            return $('header.page-header').hasClass('menu-opened');
        },

        moveBlocks: function (mode,logoLeft) {
            var headerHeight = $('.page-header').outerHeight();

            switch (mode) {
                case 'mobile':
                    if (logoLeft) {
                        this.moveSingleBlock('.logo','append','.header-central-wrapper');
                        this.moveSingleBlock('.shipping-to-link','append','#mobile-menu-bottom-container .mobile-menu-bottom-container-left');
                        this.moveSingleBlock('#switcher-language','append','#mobile-menu-bottom-container .mobile-menu-bottom-container-right');
                        this.moveSingleBlock('nav.navigation','append','#mobile-menu-centeral-container');
                        this.moveSingleBlock('.search-wrapper','append','.header-left-wrapper');
                        this.moveSingleBlock('.account-wrapper','append','#mobile-menu-top-container .mobile-menu-top-container-left');
                    } else {
                        this.moveSingleBlock('.shipping-to-link','append','#mobile-menu-bottom-container .mobile-menu-bottom-container-left');
                        this.moveSingleBlock('#switcher-language','append','#mobile-menu-bottom-container .mobile-menu-bottom-container-right');
                        this.moveSingleBlock('.header-switch-contact','append','#mobile-menu-top-container .mobile-menu-top-container-right');
                        this.moveSingleBlock('nav.navigation','append','#mobile-menu-centeral-container');
                        this.moveSingleBlock('.search-wrapper','append','.header-left-wrapper');
                        this.moveSingleBlock('.account-wrapper','append','#mobile-menu-top-container .mobile-menu-top-container-left');
                    }
                    break;
                case 'desktop':
                    if (logoLeft) {
                        this.moveSingleBlock('.logo','append','.header-left-wrapper');
                        this.moveSingleBlock('.shipping-to-link','prepend','.footer bottom-content .footer-user-menu');
                        this.moveSingleBlock('#switcher-language','append','.footer bottom-content .footer-user-menu');
                        this.moveSingleBlock('nav.navigation','append','.header-central-wrapper');
                        this.moveSingleBlock('.account-wrapper','prepend','.header-right-wrapper');
                        this.moveSingleBlock('.search-wrapper','prepend','.header-right-wrapper');
                    } else {
                        this.moveSingleBlock('.shipping-to-link','prepend','.header-left-wrapper');
                        this.moveSingleBlock('#switcher-language','append','.header-left-wrapper');
                        this.moveSingleBlock('.header-switch-contact','append','.header-left-wrapper');
                        this.moveSingleBlock('nav.navigation','append','.page-header');
                        this.moveSingleBlock('.account-wrapper','prepend','.header-right-wrapper');
                        this.moveSingleBlock('.search-wrapper','prepend','.header-right-wrapper');
                    }
                    break;

                default:
                    console.log('Unmatched mode ' + mode + ' on moveBlocks function');
                    return;
            }
        },

        moveSingleBlock: function(block, method, destination) {

            if ($(destination).children(block).length === 0) {

                switch (method) {

                    case 'append':
                        $(block).appendTo(destination);
                        $(destination).children(block).show();
                        break;

                    case 'prepend':
                        $(block).prependTo(destination);
                        $(destination).children(block).show();
                        break;

                    case 'insertAfter':
                        $(block).insertAfter(destination);
                        $(destination).children(block).show();
                        break;

                    case 'insertBefore':
                        $(block).insertBefore(destination);
                        $(destination).children(block).show();
                        break;

                    default:
                        console.log('Unmatched method ' + method + ' on moveSingleBlocks function');
                        return;
                }
            }
        },

        calcMobileMenuHeight: function() {
            var mobileMenuHeight = (window.innerHeight - window.headerHeight);
            $('.mobile-menu-dialog').height(mobileMenuHeight);
        },

        calcHeaderHeight: function() {
            window.headerHeight = $('header.page-header').outerHeight();
        },

        chekIfinView: function () {

            window.animation_elements = $('.animation-element:not(.in-view)');

            if (window.animation_elements.length === 0) {
                return;
            }

            var windowEl                = $(w),
                window_height           = windowEl.height(),
                window_top_position     = windowEl.scrollTop(),
                scrollDown              = false,
                st                      = window_top_position,
                window_bottom_position  = window_top_position + window_height,
                element_top_position    ;

            if (st >= w.lastScrollTop) {
                scrollDown = true;
            }

            window.lastScrollTop = st;

            $.each(window.animation_elements, function () {

                if ($(this).hasClass("in-view")) {
                    return;
                }

                element_top_position = $(this).offset().top;
                if (element_top_position <= window_bottom_position) {
                    $(this).addClass('in-view');
                }

                window.animation_elements = $('.animation-element:not(.in-view)');
            });
        }


    });

    return $.filoblu.layout;
});
